/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.ejb;

import org.objectweb.speedo.AbstractSpeedo;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.lib.Personality;
import org.objectweb.speedo.pm.ejb.api.EJBPOManagerFactoryItf;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceContextType;
import javax.persistence.PersistenceException;
import javax.persistence.spi.PersistenceProvider;
import javax.persistence.spi.PersistenceUnitInfo;

public class EJBSpeedo 
    extends AbstractSpeedo 
    implements PersistenceProvider, EJBPOManagerFactoryItf {

    public EJBSpeedo() throws Throwable {
        super();
    }

    /**
     * It creates and initializes a real POManagerFactory with
     * Julia (Fractal implementation).
     */
    public EJBSpeedo(Map props) throws Throwable {
        super(props);
    }

    protected void throwUserException(String msg) {
        throw new PersistenceException(msg);
    }

    private Map getSpeedoPropertiesFromFile(String fileName) {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        InputStream is = loader.getResourceAsStream(fileName);
        if (is == null) {
            return null;
        }
        Properties props = new Properties();
        try {
            props.load(is);
        } catch (IOException e) {
            throw new PersistenceException("Error during the file reading: " + fileName);
        }
        return new HashMap(props);
    }

	protected boolean isOptimisticTransaction(Map props) {
		// TODO Auto-generated method stub
		return false;
	}
	
    // IMPLEMENTS THE INTERFACE PersistenceProvider //
    //----------------------------------------------//
    
    public EntityManagerFactory createEntityManagerFactory(String name, Map props) {
        if (props == null || props.size() == 0) {
            //load file from the the emf name
            props = getSpeedoPropertiesFromFile("speedo-" + name + ".properties");
            if (props == null || props.size() == 0) {
                return null;
            }
        }
        try {
            return new EJBSpeedo(props);
        } catch (Throwable e) {
            throw new PersistenceException(e.getMessage());
        }
    }
    

    public EntityManagerFactory createContainerEntityManagerFactory(
            PersistenceUnitInfo pi) {
        if (!SpeedoProperties.VENDOR_NAME.equals(pi.getPersistenceProviderClassName())) {
            return null;
        }
        //TODO: support direct reference to Datasource
        //Object ds1 = pi.getJtaDataSource();
        //TODO: support non JTA datasource for optimistic concurrency
        //Object ds2 = pi.getNonJtaDataSource();
        // TODO implements createEntityManagerFactory(PersistenceInfo)
        return createEntityManagerFactory(
                pi.getPersistenceUnitName(), 
                new HashMap(pi.getProperties()));
    }

    // IMPLEMENTS THE INTERFACE EJBPOManagerFactoryItf //
    //-------------------------------------------------//
    
    public Personality getPersonality() {
        return Personality.EJB;
    }
    
    public void close() {
        ((EJBPOManagerFactoryItf) delegate).close();
    }

    public EntityManager createEntityManager() {
        return ((EJBPOManagerFactoryItf) delegate).createEntityManager();
    }

    public EntityManager createEntityManager(PersistenceContextType pct) {
        return ((EJBPOManagerFactoryItf) delegate).createEntityManager(pct);
    }

    public EntityManager getEntityManager() {
        return ((EJBPOManagerFactoryItf) delegate).getEntityManager();
    }

    public boolean isOpen() {
        return ((EJBPOManagerFactoryItf) delegate).isOpen();
    }

	
}
