/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.genclass.merger;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.speedo.generation.enhancer.common.LoggedClassVisitor;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.HashSet;
import java.util.Set;

/**
 * This visitor checks if the class is abstract.
 * 
 * @author S.Chassande-Barrioz
 */
public class GenClassAnalyzer extends LoggedClassVisitor {

    GCInfo gc;
    
    public GenClassAnalyzer(Logger logger, GCInfo gci) {
        super(logger);
        this.gc = gci;
    }

    public void visit(final int version, 
            final int access,
            final String name,
            final String supername,
            final String[] interfaces,
            final String sourceFile) {
        gc.isAbstract = (access & Constants.ACC_ABSTRACT) != 0;
        gc.setSuperName(supername);
        gc.itfs = interfaces;
    }
    /**
     * Add only non abstract method. Avoid method duplication
     */
    public CodeVisitor visitMethod(final int access,
            final String name,
            final String desc,
            final String[] exceptions,
            final Attribute attrs) {
        String method = name + " " +  desc;
        if ((access & Constants.ACC_ABSTRACT) == 0) {
            gc.nonAbstractMethods.add(method);
        }
        return null;
    }
}
