/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.enhancer.common;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.CodeVisitor;
import org.objectweb.speedo.lib.Personality;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.HashSet;
import java.util.Set;

/**
 * Is a simple visitor verifying if duplicated method are generated. When 
 * duplicated method is found a warning is logged.
 * 
 * @author S.Chassande-Barrioz
 */
public class DuplicatedMethodVerifier extends LoggedClassAdapter {

    /**
     * Set of methods already found.
     */
    private Set methods = new HashSet();
    
    public DuplicatedMethodVerifier(ClassVisitor classVisitor, Logger logger, Personality p) {
        super(classVisitor, p, logger);
    }
    
    /**
     * Internal representation of the method declaration
     */
    private static class Meth {
        public String name;
        public String desc;
        public int access;
        public Meth(int a, String n, String d) {
            access = a; name = n; desc = d;
        }
        public boolean equals(Object o) {
            return (o instanceof Meth) 
                && ((Meth) o).name.equals(name)
                && ((Meth) o).desc.equals(desc)
                ;
        }
        public int hashCode() {
            return name.hashCode();
        }
        public String toString() {
            return access + name + desc;  
        }
    }
    
    public CodeVisitor visitMethod(final int access,
               final String name,
               final String desc,
               final String[] exceptions,
               final Attribute attrs) {
        Meth m = new Meth(access, name, desc);
        if (methods.contains(m)) {
            logger.log(BasicLevel.WARN, "Duplicate method: '" + m);
        } else {
            methods.add(m);
        }
        return cv.visitMethod(access, name, desc, exceptions, attrs);
    }
}
