/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.generation.enhancer.common;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.speedo.lib.Personality;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.HashSet;
import java.util.Set;

/**
 * This visistor records the interfaces specified during visits. Each time
 * a class is visited, all registered interfaces are given to the next visitor. 
 *
 * @author S.Chassande-Barrioz
 */
public class InterfaceAgregatorVisitor extends LoggedClassAdapter {

    /**
     * The set of found interfaces
     */
    private final Set itfs; 
    
    public InterfaceAgregatorVisitor(ClassVisitor classVisitor, Logger logger,
            String className, Personality p) {
        super(classVisitor, p, logger);
        this.itfs = new HashSet();
    }

    public void visit(final int version, final int access, final String name,
            final String superName, final String[] interfaces,
            final String sourceFile) {
        if (interfaces != null) {
            //Register the found interfaces
            for (int i = 0; i < interfaces.length; ++i) {
                if (!itfs.contains(interfaces[i])) {
                    if (debug) {
                        logger.log(BasicLevel.DEBUG, "Add the interface '"
                                + interfaces[i] + "'");
                    }
                    itfs.add(interfaces[i]);
                }
            }
        }
        //pass all found interfaces
        cv.visit(version, access, name, superName, 
                (String[]) itfs.toArray(new String[itfs.size()]), sourceFile);
    }
}
