/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoIndex;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.sequence.lib.SpeedoSequence;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;

/**
 * Define a speedo meta information visitor. The visitors can be chained by the
 * setNext method.
 *
 * @author S.Chassande-Barrioz
 */
public interface MetaInfoVisitor {

    void visitCompilerParameter(SpeedoCompilerParameter scp) throws SpeedoException;

    void visitXml(SpeedoXMLDescriptor xml) throws SpeedoException;

    void visitPackage(SpeedoPackage sp) throws SpeedoException;

    void visitClass(SpeedoClass sc) throws SpeedoException;

    void visitEndClass(SpeedoClass sc) throws SpeedoException;
    
    void visitSequence(SpeedoSequence ss, SpeedoPackage sp) throws SpeedoException;
    
    void visitIndex(SpeedoIndex si, SpeedoPackage sp) throws SpeedoException;

    void visitField(SpeedoField sf) throws SpeedoException;

    void visitExtension(SpeedoExtension se) throws SpeedoException;
    
    void visitEnd();

    MetaInfoVisitor getNext();

    void setNext(MetaInfoVisitor next);
}
