/**
 * perseus/connector: this is an implementation of some JCA-related technologies
 * (resource adapters and managers) for the ObjectWeb consortium.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: speedo@objectweb.org
 *
 */

package org.objectweb.speedo.jca;

import javax.resource.ResourceException;

/**
 * Describes the accessors/mutators for each of the properties
 * that are supported by the JDO Resource Adapter.
 * @author  P. Dechamboux
 */
public interface SpeedoAttributeController {
    /**
     * Gives access to the name of the property file for initializing the
     * underlying JDO implementation.
     * @return  The name of the property file.
     */
    String getPropertyFile();

    /**
     * Assigns to this JDO connector the name of the property file for
     * initializing the underlying JDO implementation.
     * @param pf  The name of the property file.
     */
    void setPropertyFile(String pf);

    /**
     * Gives access to the JNDI name of the transaction manager
     *  (javax.transaction.TransactionManager).
     * @return the JNDI name of the transaction manager
     */
    String getTransactionManagerJNDIName();

    /**
     * Assigns the JNDI name of the transaction manager
     *  (javax.transaction.TransactionManager).
     * @param jndiname is the JNDI name of the transaction manager
     */
    void setTransactionManagerJNDIName(String jndiname) throws ResourceException;
}
