/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.speedo.jdo;

import javax.jdo.JDODataStoreException;
import javax.jdo.JDODetachedFieldAccessException;
import javax.jdo.JDOException;
import javax.jdo.JDOFatalDataStoreException;
import javax.jdo.JDOFatalException;
import javax.jdo.JDOUserException;

import org.objectweb.speedo.lib.Personality;
import org.objectweb.speedo.pm.jdo.api.JDOPOManagerFactoryItf;
import org.objectweb.speedo.pm.jdo.api.JDOPOManagerItf;

public final class JDOPersonality extends Personality {
	
	
	private static final long serialVersionUID = 5463434343545L;
	
	public RuntimeException newRuntimeException() {
    	return new JDOException();
    }
    public RuntimeException newRuntimeException(String msg) {
    	return new JDOException(msg);
    }
    public RuntimeException newRuntimeException(String msg, Throwable[] nested) {
    	return new JDOException(msg, nested);
    }
    public RuntimeException newRuntimeException(String msg, Throwable nested) {
    	return new JDOException(msg, nested);
    }
    public RuntimeException newRuntimeException(String msg, Object failed) {
    	return new JDOException(msg, failed);
    }
    public RuntimeException newRuntimeException(String msg, Throwable[] nested, Object failed) {
    	return new JDOException(msg, nested, failed);
    }
    public RuntimeException newRuntimeException(String msg, Throwable nested, Object failed) {
    	return new JDOException(msg, nested, failed);
    }
    public RuntimeException newRuntimeException(Throwable nested) {
    	return new JDOException("", nested);
    }
    public Class getRuntimeExceptionClass() {
    	return JDOException.class;
    }
	
    public RuntimeException newUserRuntimeException() {
    	return new JDOUserException();
    }
    public RuntimeException newUserRuntimeException(String msg) {
    	return new JDOUserException(msg);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable[] nested) {
    	return new JDOUserException(msg, nested);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable nested) {
    	return new JDOUserException(msg, nested);
    }
    public RuntimeException newUserRuntimeException(String msg, Object failed) {
    	return new JDOUserException(msg, failed);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable[] nested, Object failed) {
    	return new JDOUserException(msg, nested, failed);
    }
    public RuntimeException newUserRuntimeException(String msg, Throwable nested, Object failed) {
    	return new JDOUserException(msg, nested, failed);
    }
    public RuntimeException newUserRuntimeException(Throwable nested) {
    	return new JDOUserException("", nested);
    }
    public Class getUserRuntimeExceptionClass() {
    	return JDOUserException.class;
    }
	
    public RuntimeException newFatalRuntimeException() {
    	return new JDOFatalException();
    }
    public RuntimeException newFatalRuntimeException(String msg) {
    	return new JDOFatalException(msg);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable[] nested) {
    	return new JDOFatalException(msg, nested);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable nested) {
    	return new JDOFatalException(msg, nested);
    }
    public RuntimeException newFatalRuntimeException(String msg, Object failed) {
    	return new JDOFatalException(msg, failed);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable[] nested, Object failed) {
    	return new JDOFatalException(msg, nested, failed);
    }
    public RuntimeException newFatalRuntimeException(String msg, Throwable nested, Object failed) {
    	return new JDOFatalException(msg, nested, failed);
    }
    public RuntimeException newFatalRuntimeException(Throwable nested) {
    	return new JDOFatalException("", nested);
    }
    public Class getFatalRuntimeExceptionClass() {
    	return JDOFatalException.class;
    }
	public RuntimeException newFatalDataStoreRuntimeException() {
		return new JDOFatalDataStoreException();
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Object failed) {
		return new JDOFatalDataStoreException(msg, failed);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable nested, Object failed) {
		return new JDOFatalDataStoreException(msg, nested);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable nested) {
		return new JDOFatalDataStoreException(msg, nested);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable[] nested, Object failed) {
		return new JDOFatalDataStoreException(msg, nested);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg, Throwable[] nested) {
		return new JDOFatalDataStoreException(msg, nested);
	}
	public RuntimeException newFatalDataStoreRuntimeException(String msg) {
		return new JDOFatalDataStoreException(msg);
	}
	public RuntimeException newFatalDataStoreRuntimeException(Throwable nested) {
		return new JDOFatalDataStoreException("", nested);
	}
	public RuntimeException newDataStoreRuntimeException(String msg) {
		return new JDODataStoreException(msg);
	}
	public RuntimeException newDataStoreRuntimeException(String msg, Throwable nested) {
		return new JDODataStoreException(msg, nested);
	}
	
	public Class getDetachedFieldAccessRuntimeExceptionClass() {
		return JDODetachedFieldAccessException.class;
	}
	public Class getPOMClass() {
		return JDOPOManagerItf.class;
	}
	public Class getPOMFClass() {
		return JDOPOManagerFactoryItf.class;
	}
}
