/**
 * Copyright (C) 2001-2005 France Telecom R&D
 */
package org.objectweb.speedo.jmx.mbeans;

import org.objectweb.perseus.cache.api.CacheAttributeController;
import org.objectweb.perseus.cache.api.CacheException;
import org.objectweb.perseus.cache.api.CacheManager;
import org.objectweb.perseus.cache.api.UnbindManager;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.pm.api.POManagerFactoryItf;
import org.objectweb.speedo.pm.jdo.api.JDOPOManagerFactoryItf;

import java.util.Collection;

/**
 * 
 *
 * @author chassase
 */
public class Cache  implements CacheMBean {

    JDOPOManagerFactoryItf pmf;
    JormFactory jf;
    CacheAttributeController ca;
    UnbindManager um; 
    CacheManager cm;
    
    
    public Cache(CacheAttributeController ca, 
            UnbindManager um, 
            CacheManager cm,
            JDOPOManagerFactoryItf pmf,
            JormFactory jf) {
        this.ca = ca;
        this.um = um;
        this.cm = cm;
        this.pmf = pmf;
        this.jf = jf;
    }
    public String getCache_AutoCleanSize() {
        return ca.getAutoCleanSize();
    }
    public String getCache_AutoCleanThreshold() {
        return ca.getAutoCleanThreshold();
    }
    public Collection getCache_ObjectIdentifiers() {
        return ca.getCurrentEntryIdentifiers();
    }
    public int getCache_CurrentSize() {
        return ca.getCurrentSize();
    }
    public int getCache_MaxObjects() {
        return ca.getMaxObjects();
    }
    public void setCache_AutoCleanSize(String size) {
        ca.setAutoCleanSize(size);
    }
    public void setCache_AutoCleanThreshold(String size) {
        ca.setAutoCleanThreshold(size);
    }
    public void setCache_MaxObjects(int size) throws IllegalArgumentException,
            CacheException {
    	ca.setMaxObjects(size);
    }

    public void evictAll(String classname, boolean subclass) {
        ClassLoader cl = jf.getClassLoader(classname);
        if (cl != null) {
            try {
                pmf.getDataStoreCache().evictAll(cl.loadClass(classname), subclass);
            } catch (ClassNotFoundException e) {
            }
        }
    }
    
    public void evictAll() {
        pmf.getDataStoreCache().evictAll();
    }

    public void pinAll(String classname, boolean subclass) {
        ClassLoader cl = jf.getClassLoader(classname);
        if (cl != null) {
            try {
                pmf.getDataStoreCache().pinAll(cl.loadClass(classname), subclass);
            } catch (ClassNotFoundException e) {
            }
        }
    }
    
    public void unpinAll(String classname, boolean subclass) {
        ClassLoader cl = jf.getClassLoader(classname);
        if (cl != null) {
            try {
                pmf.getDataStoreCache().unpinAll(cl.loadClass(classname), subclass);
            } catch (ClassNotFoundException e) {
            }
        }
    }
}
